import argparse
from overrides import overrides
from typing import List

from sacrerouge.commands import MetricSetupSubcommand
from sacrerouge.data import MetricsDict
from sacrerouge.data.types import ReferenceType, SummaryType
from sacrerouge.metrics import Metric, ReferenceBasedMetric

try:
    from bert_score import score, create_idf_dict
except ImportError:
    BERTSCORE_INSTALLED = False

    @Metric.register('bertscore')
    class BertScore(ReferenceBasedMetric):
        def __init__(self, *args, **kwargs):
            pass

        def score_multi_all(self, *args, **kwargs):
            raise NotImplementedError('Please install the "bert_score" python library library to use BertScore')
else:
    BERTSCORE_INSTALLED = True

    @Metric.register('bertscore')
    class BertScore(ReferenceBasedMetric):
        def __init__(self,
                     model_type: str = None,
                     num_layers: int = None,
                     idf: bool = False,
                     nthreads: int = 4,
                     batch_size: int = 64,
                     lang: str = 'en',
                     verbose: bool = False) -> None:
            super().__init__()
            self.model_type = model_type
            self.num_layers = num_layers
            self.idf = idf
            self.nthreads = nthreads
            self.batch_size = batch_size
            self.lang = lang
            self.verbose = verbose

        def _flatten_summary(self, summary: SummaryType) -> str:
            if isinstance(summary, list):
                return ' '.join(summary)
            return summary

        def _get_unique_references(self, references_list: List[List[str]]) -> List[str]:
            unique_references = set()
            for references in references_list:
                for reference in references:
                    unique_references.add(reference)
            return list(unique_references)

        def _run(self,
                 summaries_list: List[List[SummaryType]],
                 references_list: List[List[SummaryType]]) -> List[List[MetricsDict]]:
            summaries_list = [[self._flatten_summary(summary) for summary in summaries] for summaries in summaries_list]
            references_list = [[self._flatten_summary(reference) for reference in references] for references in
                               references_list]

            # Create the candidate and reference lists for passing to the scoring function
            input_candidates = []
            input_references = []
            empty_inputs = set()
            for i, (summaries, references) in enumerate(zip(summaries_list, references_list)):
                for j, summary in enumerate(summaries):
                    if len(summary) == 0:
                        empty_inputs.add((i, j))
                    else:
                        input_candidates.append(summary)
                        input_references.append(references)

            # If we are using IDF weighting, then the default script will create
            # the IDF score based on all of the references, but we could potentially
            # have duplicate references, which would make "incorrect" counts. Instead,
            # create the idf dict based on unique references
            if self.idf:
                unique_references = self._get_unique_references(references_list)
                idf = create_idf_dict(unique_references, model_type=self.model_type, nthreads=self.nthreads,
                                      lang=self.lang)
            else:
                idf = False

            # Score the summaries
            precisions, recalls, f1s = score(input_candidates, input_references,
                                             model_type=self.model_type,
                                             num_layers=self.num_layers,
                                             idf=idf,
                                             nthreads=self.nthreads,
                                             batch_size=self.batch_size,
                                             lang=self.lang,
                                             verbose=self.verbose)

            # Remap the scores to the summaries
            index = 0
            metrics_lists = []
            for i, summaries in enumerate(summaries_list):
                metrics_lists.append([])
                for j, summary in enumerate(summaries):
                    if (i, j) in empty_inputs:
                        precision, recall, f1 = 0.0, 0.0, 0.0
                    else:
                        precision = precisions[index].item()
                        recall = recalls[index].item()
                        f1 = f1s[index].item()
                        index += 1

                    metrics_lists[-1].append(MetricsDict({
                        'bertscore': {
                            'precision': precision,
                            'recall': recall,
                            'f1': f1,
                        }
                    }))

            return metrics_lists

        def score_multi_all(self,
                            summaries_list: List[List[SummaryType]],
                            references_list: List[List[ReferenceType]]) -> List[List[MetricsDict]]:
            return self._run(summaries_list, references_list)


@MetricSetupSubcommand.register('bertscore')
class BertScoreSetupSubcommand(MetricSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the BERTScore metric'
        self.parser = parser.add_parser('bertscore', description=description, help=description)
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        try:
            import bert_score
            print('BertScore setup success')
        except ImportError:
            print('Please pip install "bert_score" to complete setup')


import unittest

from sacrerouge.common.testing.metric_test_cases import DocumentBasedMetricTestCase
from sacrerouge.common.testing.util import sacrerouge_command_exists
from sacrerouge.metrics import Blanc
from sacrerouge.metrics.blanc import BLANC_INSTALLED


@unittest.skipIf(not BLANC_INSTALLED, '"blanc" not installed')
class TestBlanc(DocumentBasedMetricTestCase):
    def test_command_exists(self):
        assert sacrerouge_command_exists(['blanc'])

    def test_setup_command_exists(self):
        assert sacrerouge_command_exists(['setup-metric', 'blanc'])

    def test_blanc_help_regression(self):
        # Tests the examples from the official github repo, but they do not match the scores there and I don't know why
        metric = Blanc(blanc_type='blanc_help', random_seed=4)
        documents_list = [
            ['Jack drove his minivan to the bazaar to purchase milk and honey for his large family.'],
            ['As Jill started taking a walk in the park, she certainly noticed that the trees were extra green this year.']
        ]
        summaries_list = [
            ['Jack bought milk and honey.', 'Jack drove to the bazaar in a minivan'],
            ['Jill saw green trees in the park.', 'The trees were green.']
        ]
        expected_scores_list = [
            [0.2222222222222222, 0.2222222222222222],
            [0.14285714285714285, 0.14285714285714285]
        ]
        actual_scores_list = metric.score_multi_all(summaries_list, documents_list)

        assert len(expected_scores_list) == len(actual_scores_list)
        for expected_scores, actual_scores in zip(expected_scores_list, actual_scores_list):
            assert len(expected_scores) == len(actual_scores)
            for expected, actual in zip(expected_scores, actual_scores):
                self.assertAlmostEqual(expected, actual['blanc_help'], places=4)

    def test_blanc_tune_repo(self):
        # Tests the examples from the official github repo. The scores don't match because they change based
        # on the random seed and the repo does not set the random seed
        metric = Blanc(blanc_type='blanc_tune', gap=6)
        documents_list = [
            ['Jack drove his minivan to the bazaar to purchase milk and honey for his large family.'],
            ['As Jill started taking a walk in the park, she certainly noticed that the trees were extra green this year.']
        ]
        summaries_list = [
            ['Jack bought milk and honey.', 'Jack drove to the bazaar in a minivan'],
            ['Jill saw green trees in the park.', 'The trees were green.']
        ]
        # The github repo shows the last 2 numbers are negative in some spots, but positive in others. I think
        # they're supposed to be positive
        expected_scores_list = [
            [0.1111111111111111, 0.2222222222222222],
            [0.14285714285714285, 0.14285714285714285]
        ]
        actual_scores_list = metric.score_multi_all(summaries_list, documents_list)

        assert len(expected_scores_list) == len(actual_scores_list)
        for expected_scores, actual_scores in zip(expected_scores_list, actual_scores_list):
            assert len(expected_scores) == len(actual_scores)
            for expected, actual in zip(expected_scores, actual_scores):
                self.assertAlmostEqual(expected, actual['blanc_tune'], places=4)



import argparse
import numpy as np
import random
from overrides import overrides
from typing import List, Union

from sacrerouge.commands import MetricSetupSubcommand
from sacrerouge.data import MetricsDict
from sacrerouge.data.types import DocumentType, SummaryType
from sacrerouge.metrics import DocumentBasedMetric, Metric


try:
    from blanc import BlancHelp, BlancTune
except ImportError:
    BLANC_INSTALLED = False

    class Blanc(DocumentBasedMetric):
        def __init__(self, *args, **kwargs):
            pass

        def score_multi_all(self, *args, **kwargs):
            raise NotImplementedError('Please install the "blanc" python library library to use Blanc')
else:
    BLANC_INSTALLED = True

    import torch

    @Metric.register('blanc')
    class Blanc(DocumentBasedMetric):
        def __init__(self,
                     blanc_type: str = 'blanc_help',
                     device: str = 'cuda',
                     gap: int = 2,
                     inference_batch_size: int = 24,
                     finetune_mask_evenly: bool = False,
                     finetune_batch_size: int = 24,
                     show_progress_bar: bool = False,
                     random_seed: int = 1) -> None:
            super().__init__()
            self.name = blanc_type

            # The blanc main class sets the random seed. It is not done in BlancTune/BlancHelp,
            # so we also do it here. 1 is their default value
            random.seed(random_seed)
            np.random.seed(random_seed)
            torch.manual_seed(random_seed)

            if blanc_type == 'blanc_tune':
                self.blanc = BlancTune(device=device, gap=gap, inference_batch_size=inference_batch_size,
                                       finetune_mask_evenly=finetune_mask_evenly,
                                       finetune_batch_size=finetune_batch_size,
                                       show_progress_bar=show_progress_bar)
            elif blanc_type == 'blanc_help':
                self.blanc = BlancHelp(device=device, gap=gap, inference_batch_size=inference_batch_size,
                                       show_progress_bar=show_progress_bar)
            else:
                raise Exception(f'Unknown BLANC type: {blanc_type}')

        def _flatten(self, text: Union[DocumentType, SummaryType]) -> str:
            if isinstance(text, list):
                return ' '.join(text)
            return text

        def score_multi_all(self,
                            summaries_list: List[List[SummaryType]],
                            documents_list: List[List[DocumentType]]) -> List[List[MetricsDict]]:
            # For now, the implementation of the BLANC library looks like it only allows for single-document summaries
            documents = []
            for docs in documents_list:
                assert len(docs) == 1, 'BLANC only supports evaluating single-document summaries'
                documents.append(self._flatten(docs[0]))

            summaries_list = [[self._flatten(summary) for summary in summaries] for summaries in summaries_list]

            scores_list = self.blanc.eval_summaries_for_docs(documents, summaries_list)
            metrics_lists = []
            for scores in scores_list:
                metrics_lists.append([])
                for score in scores:
                    metrics_lists[-1].append(MetricsDict({self.name: score}))
            return metrics_lists


@MetricSetupSubcommand.register('blanc')
class BlancSetupSubcommand(MetricSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the BLANC metric'
        self.parser = parser.add_parser('blanc', description=description, help=description)
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        try:
            import blanc
            print('BLANC setup success')
        except ImportError:
            print('Please pip install "blanc" to complete setup')
